//
//  DataHelper.swift
//  MicrocomFirstApp
//
//  Created by Microcom Developer on 9/22/17.
//  Copyright © 2017 Cameron Wrabel. All rights reserved.
//

import Foundation
import CoreFoundation


class DataHelper {
    
    // converts a string to a uint8 byte array
    static func String2UInt8(str: String) -> [UInt8]{
        return [UInt8]("\(str)".utf8)
    }
    
    // converts uint8 byte array to string
    static func UInt82String(bytes: [UInt8]) -> String{
        if let string = String(data: Data(bytes), encoding: .utf8){
            return string
        }else{
            print("Not a valid UTF-8 sequence\n")
            return ""
        }
    }
    
    static func validateIPv4AddressString(_ ipString: String) -> Bool {
        //var isValid = true
        
        var sin = sockaddr_in()
        var sin6 = sockaddr_in6()
        
        if ipString.withCString({ cstring in inet_pton(AF_INET6, cstring, &sin6.sin6_addr) }) == 1 {
            // IPv6 peer.
            return true
        }
        else if ipString.withCString({ cstring in inet_pton(AF_INET, cstring, &sin.sin_addr) }) == 1 {
            // IPv4 peer.
            return true
        }
        
        return false
    }
    
    
    static func validatePortString(_ portString: String) -> Bool {
        //var isValid = true
        
        // if the string is longer than 5 characters, it cannot be used to correctly represent the range of valid ports
        if (portString.count < 6) && (portString.count > 0) {
            
            // try to convert the string to an integer
            let portInt:Int? = Int(portString)
            
            // if it is not nil, the conversion was successful
            if portInt != nil {
                
                // check that the converted integer is within the valid port range
                if (portInt! < 65535) && (portInt! > 0) {
                    return true
                }
                
            }
        }
        
        return false
        //return isValid
        
    }
}

