//
//  FileIOHelper.swift
//  MicrocomFirstApp
//
//  Created by Microcom Developer on 9/22/17.
//  Copyright © 2017 Cameron Wrabel. All rights reserved.
//

import Foundation
import CoreFoundation


extension String {
    func appendLineToURL(fileURL: URL) throws {
        try (self + "\n").appendToURL(fileURL: fileURL)
    }
    
    func appendToURL(fileURL: URL) throws {
        let data = self.data(using: String.Encoding.utf8)!
        try data.append(fileURL: fileURL)
    }
}

extension Data {
    func append(fileURL: URL) throws {
        if let fileHandle = FileHandle(forWritingAtPath: fileURL.path) {
            defer {
                fileHandle.closeFile()
            }
            fileHandle.seekToEndOfFile()
            fileHandle.write(self)
            
            fileHandle.closeFile() // XXX
        }
        else {
            try write(to: fileURL, options: .atomic)
        }
    }
}

class FileIOHelper {
    
    
    
    
    static func documentsDirectory() -> String {
        let documentsFolderPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .allDomainsMask, true)[0]
        return documentsFolderPath
    }
    
    static func fileInDocumentsDirectory(filename: String) -> String {
        return documentsDirectory() + "/" + filename
    }
    
    static func saveImage( image: UIImage, path: String) -> Bool {
        //let urlPath = URL(string: path)
        let pngImageData = UIImagePNGRepresentation(image)
        
        let fileURL = try! FileManager.default.url(for: .documentDirectory, in: .userDomainMask, appropriateFor: nil, create: false).appendingPathComponent("\(path)")
        
        do {
            try pngImageData?.write(to: fileURL, options: .atomic)
            return true
        } catch {
            print(error)
            return false
        }
        
        //let result = try! pngImageData?.write(to: urlPath!, options: NSData.WritingOptions.atomic)
        //return (result != nil)
    }
    
    static func writeTextToFile(text: String, path: String) -> Bool {
        
        let fileURL = try! FileManager.default.url(for: .documentDirectory, in: .userDomainMask, appropriateFor: nil, create: false).appendingPathComponent("\(path)")
        
        
        
        do{
            //try text.write(to: fileURL, atomically: false, encoding: .utf8)
            try text.appendLineToURL(fileURL: fileURL)
            return true
        }catch{
            print(error)
            return false
        }
        
        
    }
    
    static func writeIntArrayToFile( buf: [String], path: String) -> Bool {
        
        let writeArray = buf as NSArray
        
        
        do {
            try writeArray.write(toFile: path, atomically: true)
            
            return true
        } catch {
            print(error)
            return false
        }
        
    }
    
    static func loadImageFromPath(path: String) -> UIImage? {
        let image = UIImage(contentsOfFile: path)
        if image == nil {
            print("Missing image at \(path)")
        }
        print("\(path)")
        return image
    }
    
    static func removeFileFromDocumentDirectory(filename: String) -> Bool{
        let fileManager = FileManager.default
        
        do{
            try fileManager.removeItem(atPath: fileInDocumentsDirectory(filename: filename))
            return true
        }catch{
            print("Could not remove the file \(filename)")
            return false
        }
    }
    
    static func checkFileExists(filename: String)->Bool {
        let path = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)[0] as String
        let url = URL(fileURLWithPath: path)
        
        let filePath = url.appendingPathComponent(filename).path
        let fileManager = FileManager.default
        if fileManager.fileExists(atPath: filePath) {
            print("FILE AVAILABLE")
            return true
        } else {
            print("FILE NOT AVAILABLE")
            return false
        }
    }
    
}
