//
//  LDS1ImagePrint.swift
//  MicrocomFirstApp
//
//  Created by Microcom Developer on 9/22/17.
//  Copyright © 2017 Cameron Wrabel. All rights reserved.
//

import Foundation
import CoreFoundation

class LDS1ImagePrint {
    
    // media types - continuous, blackline, blowhole, diecut
    
    // form size - height and width
    
    // dispense modes/cut modes
    
    // header format variables
    // TODO: implement these in the settings page
//    var HFM : Int = 1
//    var LSX : Int = 1280
//    var LSY : Int
//    var WEB : Int = 0
//    var GAP : Int
//    var DPS : Int
//    var LCB : Int
//    var AGD : Int = 1
//    var SPG : Int
//    var OFX : Int = 0
//    var OFY : Int = 0
    
    
    // scale, center, or pad the image
    var padToPrintWidth: Bool = true
    var scaleToPrintWidth: Bool = false
    var centerImageOnTicket: Bool = true
   
    
    
    
    
    /*
     *  Prepares the MagickWand, sets the image parameters (size, scale, color, etc.)
     */
    func prepareImageForMicrocomImagePrint(fileName: String, wand: OpaquePointer!) -> Int8 {
        var exitCode: Int8 = 0
        
        // read the image data into the magickwand
        let fname = FileIOHelper.fileInDocumentsDirectory(filename: fileName)
        MagickReadImage(wand, fname)
        
        let format = String(cString: MagickGetImageFormat(wand))
        let imgHeight = MagickGetImageHeight(wand)
        let imgWidth = MagickGetImageWidth(wand)
        let newImgHeight = 1280
        
        let printWidth = 1280
        
        // create a background canvas for the image to sit on
        let backgroundCanvas = NewMagickWand() // XXX
        let pixWand = NewPixelWand() // XXX
        PixelSetColor(pixWand, "white") // XXX
        //MagickNewImage(backgroundCanvas, 1280, 1280, pixWand) // XXX
        MagickNewImage(backgroundCanvas, 1280, imgHeight, pixWand) // XXX
        
        // the offsets for centering the image about the canvas. these can be negative if an image is larger than the canvas is centered
        var xOffset: Int?
        var yOffset: Int?
        
        if(scaleToPrintWidth){
            MagickScaleImage(wand, printWidth, newImgHeight)
        }else{
            if(centerImageOnTicket){ // center the image about the canvas
                xOffset = (1280-imgWidth)/2
                
               
                yOffset = 0
                //yOffset = (1280-imgHeight)/2
            }else{
                xOffset = 0
                yOffset = 0
            }
            
            // place the image in "wand" on top of the image in "backgroundCanvas", positioned at the x and y offsets
            MagickCompositeImage(backgroundCanvas, wand, OverCompositeOp, xOffset!, yOffset!) // XXX
            
            MagickAddImage(wand, backgroundCanvas) // add the padded image to "wand"
            MagickNextImage(wand) // make the padded image the current image
            //MagickWriteImage(backgroundCanvas, FileIOHelper.fileInDocumentsDirectory(filename: "magickPaddedImage.png")) // write the image to a file

            //MagickReadImage(wand, FileIOHelper.fileInDocumentsDirectory(filename: "magickPaddedImage.png"))
            //MagickWriteImage(wand, FileIOHelper.fileInDocumentsDirectory(filename: "magickAddedImage.png"))
        }
        
        
        MagickSetFormat(wand, "png")
        
        // set the image type to bilevel
        MagickSetImageType(wand, BilevelType)
        
        MagickSetImageColorspace(wand, GRAYColorspace)
        
        ClearMagickWand(backgroundCanvas)
        ClearPixelWand(pixWand)
        DestroyMagickWand(backgroundCanvas) // destroy the background canvas wand
        DestroyPixelWand(pixWand) // destroy the pixel wand
        
        return exitCode
    }


    // returns the header file required for image printing as a byte array
    // note that this uses Windows line endings (CR/LF)
    func generateLDS1Header(width: Int, height: Int) -> [UInt8] {

        //let dispenseString = "^A1^D111\r\n" // cut after label
        
        // continuous, cut after label
        var headerStrings = [String]()
        headerStrings = ["\r\n", "^A100^D36\r\n", "^A0^D97\r\n", "^A1^D111\r\n", "^A1000^D96\r\n", "^A161^D95\r\n" , "^A161^D112\r\n" , "^A7^D175\r\n" , "^A075^D172\r\n", "^A25473^D172\r\n", "^A170^D172\r\n", "^A1971^D172\r\n" , "^A1972^D172\r\n" , "^D57\r\n" , "1,\(width),\(height),0,35,4,2,1,1,0,0\r\n" , "1,1,1,1,1\r\n" , "^D56\r\n" , "^D169\r\n" , "\((width*height/8))\r\n"]
        
        var headerBytes = [UInt8]()
        for i in 0..<headerStrings.count {
            headerBytes += DataHelper.String2UInt8(str: headerStrings[i])
        }
        
        return headerBytes
    }


    // returns the image pixel data for image printing as a byte array
    func generateLDS1ImageData(wand: OpaquePointer!, width: Int, height: Int) -> [UInt8] {
        
        // allocate the array to hold the pixels. amount of bytes to allocate is the width * height * number of channels * StorageType size (in bytes)
        var pixelPtr = UnsafeMutableRawPointer.allocate(bytes: (width*height*1*1), alignedTo: 1)
        
        MagickExportImagePixels(wand, 0, 0, width, height, "I", CharPixel, pixelPtr)
        
        var imageDataArray = [UInt8]()
        
        for i in stride(from: 0, to: (width*height), by: 8) { //outer loop offset by 8
            var byteString = ""
            var pixelValuesToDataByte = [UInt8](repeatElement(0, count: 8))
            
            for j in 0..<8 {
                let offset = pixelPtr + i + j // the pointer offset to the 8 bit window, and the j slider through that window
                
                let x = offset.load(as: UInt8.self) // load the value at this location
                
                if x != 0 { // flip the values 0 and 1. This is how the printer expects data to arrive.
                    pixelValuesToDataByte[j] = 1
                    byteString.append("0")
                }else {
                    byteString.append("1")
                }
            }
            
            //byteStringArray.append(byteString)
            imageDataArray.append(UInt8(byteString, radix:2)!)
        }
        
        pixelPtr.deallocate(bytes: (width*height), alignedTo: 1)
        
        return imageDataArray
    }


    // sends the header file to the open stream object
    func sendLDS1Header(header: [UInt8], stream: TCPSocks) -> Bool{
        var writeResult = 0 // make this zero. if the stream does not have space, it will be zero so this will accurately reflect the failure of that condition.


        //writeResult = writeBytesToStream(bytes: header)
        writeResult = stream.writeBytesToStream(bytes: header)



        if writeResult == header.count {
            return true
        }else{
            return false
        }
    }


    // sends the body of the image data -- does not wait for the image size response from printer
    func sendLDS1ImageData(body: [UInt8], stream: TCPSocks) -> Bool{
        var writeResult = 0 // make this zero. if the stream does not have space, it will be zero so this will accurately reflect the failure of that condition.

        writeResult = stream.writeBytesToStream(bytes: body)

        if writeResult == body.count {
            return true
        }else{
            return false
        }

    }


    func sendLDS1ChunkedImageData(body: [[UInt8]], stream: TCPSocks) -> Bool {
        var writeResult = 0

        for i in body {
            writeResult = stream.writeBytesToStream(bytes: i)

            if writeResult != i.count {
                print("***** sendLDS1ChunkedImageData: failed writing chunk. writeResult = \(writeResult)")
                return false
            }
        }
        return true
    }


    // perform the "managed" LDS1 image print operation
    func performLDS1ImagePrint(stream: TCPSocks!, wand: OpaquePointer!, width: Int, height: Int) -> [UInt8] {
        
        var ldsSocketStream: TCPSocks? = stream
        
        
        var headerData = generateLDS1Header(width: width, height: height)
        var imageBodyData = generateLDS1ImageData(wand: wand, width: width, height: height)
        
        //print("***** headerData is \(headerData.count) bytes long\n")
        //print("***** imageBodyData is \(imageBodyData.count) bytes long\n")
        
        //waitingForResponse = true
        //expectingSize = true
        //desiredResponse = "\(imageBodyData.count)"
        
        
        
        var imageDataChunks = chunkifyImageData(arr: imageBodyData, chunkSize: UserDefaults.standard.integer(forKey: "imageChunkSize"))
        
        
        _ = sendLDS1Header(header: headerData, stream: stream)
        
        //let bodyResult = sendLDS1ImageData(body: imageBodyData)
        _ = sendLDS1ChunkedImageData(body: imageDataChunks, stream: stream)
        
        //imageData = imageBodyData
        
        return imageBodyData
    }


    // break the image data array into small, digestable pieces so that the printer doesn't time out waiting for the image data to arrive
    func chunkifyImageData(arr: [UInt8], chunkSize: Int) -> [[UInt8]]{
        
        var imageDataChunks = [[UInt8]]()
        let arraySlices = (arr.count / chunkSize) + 1
        
        for i in 0..<arraySlices {
            
            let chunkStartingIndex = i*chunkSize  // the start of each chunk found by the chunk number and the chunk size
            var chunkEndingIndex = chunkStartingIndex + chunkSize // all chunks except the last one are guaranteed to be this size
            
            if i == (arraySlices-1) // the last chunk may be smaller than chunkSize
            {
                chunkEndingIndex = arr.count
            }
            let chunk = Array(arr[chunkStartingIndex..<chunkEndingIndex]) // this array contains a range from the image data
            imageDataChunks.append(chunk)
            //print("***** chunkifyImageData: chunk \(i) size \(imageDataChunks[i].count)")
        }
        //print("***** chunkifyImageData: \(arraySlices)")
        return imageDataChunks
    }
    
    // unused
    func makeImageMonochrome(){
        // initialize the wand environment
        MagickWandGenesis()
        // create the magick wand
        var wand = NewMagickWand()
        
        // read the given image with a c-type string
        MagickReadImage(wand, FileIOHelper.fileInDocumentsDirectory(filename: "tempImage.png"))
        
        // monochrome image
        MagickQuantizeImage(wand, 2, GRAYColorspace, 1, MagickFalse, MagickFalse)
        
        // write to temporary file
        MagickWriteImage(wand, FileIOHelper.fileInDocumentsDirectory(filename: "tempImage3.png"))
        
        //load UIImage from temporary file
        var editedImg = UIImage(contentsOfFile: FileIOHelper.fileInDocumentsDirectory(filename: "tempImage3.png"))
        
        //microcomImageView.image = editedImg
        DestroyMagickWand(wand)
    }
    
    
}
