//
//  SettingsTableViewController.swift
//  MicrocomFirstApp
//
//  Created by Microcom Developer on 7/10/17.
//  Copyright © 2017 Cameron Wrabel. All rights reserved.
//

import UIKit

class SettingsTableViewController: UITableViewController, UIPickerViewDelegate, UIPickerViewDataSource, UITextFieldDelegate {

    
    /**********************************************/
    // MARK: Outlets
    /**********************************************/

    @IBOutlet weak var discoverDevicesTableCell: UITableViewCell!
    @IBOutlet weak var headWidthTextField: SettingsUITextField!
    @IBOutlet weak var dpiTextField: SettingsUITextField!
    @IBOutlet weak var stockWidthTextField: SettingsUITextField!
    @IBOutlet weak var maintainAspectRatioSwitch: UISwitch!
    @IBOutlet weak var scaleToStockSwitch: UISwitch!
    @IBOutlet weak var udpDiscoverySwitch: UISwitch!
    @IBOutlet weak var mediaTypeTextField: SettingsUITextField!
    @IBOutlet weak var dispenseModeTextField: SettingsUITextField!
    
    // number edits
    @IBOutlet weak var advanceDistanceTextField: SettingsUITextField!
    @IBOutlet weak var advanceRetractDelayTextField: SettingsUITextField!
    @IBOutlet weak var stepsPastGapTextField: SettingsUITextField!
    @IBOutlet weak var activateGapDetectionTextField: SettingsUITextField!
    @IBOutlet weak var drmTextField: SettingsUITextField!
    @IBOutlet weak var dotToGapTextField: SettingsUITextField!
    @IBOutlet weak var blacklineBlowholeHeightTextField: SettingsUITextField!
    @IBOutlet weak var gapHeightTextField: SettingsUITextField!
    
    @IBOutlet var settingsTable: UITableView!
    
    @IBOutlet weak var editToolBar: UIToolbar!
    
    
    /**********************************************/
    // MARK: Picker collections
    /**********************************************/

    let dpiCollection: [Int] = [203, 300]
    let dpiPicker : UIPickerView! = UIPickerView()
    
    let headWidthCollection : [Int] = [448, 640, 672, 832, 960, 1280]
    let headWidthPicker : UIPickerView! = UIPickerView()
    
    let stockWidthCollection : [String] = ["1.0", "1.5", "2.0", "2.5", "3.0", "3.5", "4.0"]
    let stockWidthPicker : UIPickerView! = UIPickerView()
    
    let dispenseModeCollection : [String] = ["None","Advance when idle", "Advance every label", "Cut after label", "Cut with LPD" ]
    let dispenseModePicker : UIPickerView! = UIPickerView()
    
    let mediaTypeCollection : [String] = ["Die-Cut", "Blow Hole", "Black Line", "Continuous"]
    let mediaTypePicker : UIPickerView! = UIPickerView()

    
    

    //let toolBar = UIToolbar()
    //var doneButton = UIBarButtonItem()
    
    var currentControl : UIControl?
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        
        
        settingsTable.allowsSelection = false
        
        configurePickersForInput()
        addTargetToControls()
        
        
        initializeViewsWithPreferenceValues()
        
        //self.settingsTable.
        
        // Uncomment the following line to preserve selection between presentations
        // self.clearsSelectionOnViewWillAppear = false

        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        // self.navigationItem.rightBarButtonItem = self.editButtonItem()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    
    
    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        switch section {
        case 0:
            return 0 // discovery not implemented yet
        case 1:
            return 3
        case 2:
            return 0 // image printing settings not implemented throughout the app
        case 3:
            return 0 // media settings not yet implemented in printing operations
        default:
            return settingsTable.numberOfRows(inSection: section)
        }
    }
    

    /**********************************************/
    // MARK: Actions
    /**********************************************/

    func donePicker(sender: UIBarButtonItem) {
        
        print("bob")

        
    }
    
    @objc func settingsValuesChanged(_ sender: UIControl?){
        
        // loop through the controls and write their currently held values to the userdefaults
        
        let applicationPreferences = UserDefaults.standard
        applicationPreferences.set(Int(self.headWidthTextField.text!), forKey: "printWidth")
        applicationPreferences.set(Int(self.dpiTextField.text!), forKey: "printerDPI")
        applicationPreferences.set(stockWidthTextField.text, forKey: "stockWidth")
        //applicationPreferences.set(50000, forKey: "imageChunkSize")
        applicationPreferences.set(self.udpDiscoverySwitch.isOn, forKey: "udpDiscovery")
        applicationPreferences.set(self.scaleToStockSwitch.isOn, forKey: "scaleImageToFitStock")
        //applicationPreferences.set(true, forKey: "padImageToFitPrintWidth")
        applicationPreferences.set(self.maintainAspectRatioSwitch.isOn, forKey: "maintainAspectRatio")
        applicationPreferences.set(mediaTypeTextField.text, forKey: "mediaType")
        applicationPreferences.set(dispenseModeTextField.text, forKey: "dispenseMode")
        applicationPreferences.set(Int(self.advanceDistanceTextField.text!), forKey: "advanceRetractDistance") // advance retract distance
        applicationPreferences.set(Int(self.advanceRetractDelayTextField.text!), forKey: "advanceRetractDelay")// advance retract delay
        
        applicationPreferences.set(Int(self.stepsPastGapTextField.text!), forKey: "stepsPastGap")// SPG
        applicationPreferences.set(Int(self.activateGapDetectionTextField.text!), forKey: "activateGapDetection")//AGD
        applicationPreferences.set(Int(self.drmTextField.text!), forKey: "distanceToRegistrationOffset")//DRM
        applicationPreferences.set(Int(self.dotToGapTextField.text!), forKey: "dotToGap_BlackLine")//D2G
        
    }
    
    @IBAction func textField_EditingDidBegin(_ sender: UITextField, forEvent event: UIEvent) {
        
        switch sender {
            case advanceDistanceTextField,
             advanceRetractDelayTextField,
             dotToGapTextField,
             stepsPastGapTextField,
             activateGapDetectionTextField,
             drmTextField,
             blacklineBlowholeHeightTextField,
             gapHeightTextField:
                sender.inputAccessoryView = editToolBar
            break
        default:
            break
        }

        
        sender.textColor = SettingsUITextField.editingTextColorPurple
    }
    
    @IBAction func textField_EndEdit(_ sender: UITextField, forEvent event: UIEvent) {
        sender.textColor = SettingsUITextField.defaultTextColorBlack
        
        //doneButtonClicked(_ sender:)
    }
    
    
    @IBAction func udpSwitchChange(_ sender: UISwitch) {
        print("bob")
    }

    
    @IBAction func doneButtonClicked(_ sender: Any) {
        view.endEditing(true)
    }
    
    
    /**********************************************/
    // MARK: Text Field Delegate
    /**********************************************/

//    public func textFieldShouldBeginEditing(_ textField: UITextField) -> Bool {
//
//        //textField.inputAccessoryView = editToolBar
//        return false
//    }
    
    public func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        return false
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool{
        // Hide the keyboard
        textField.resignFirstResponder()
        return true
    }
       
    
    /**********************************************/
    // MARK: Picker View Data Source
    /**********************************************/

    
    // returns the number of 'columns' to display.
    public func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    
    
    // returns the # of rows in each component..
    public func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        

        
        switch pickerView {
        case headWidthPicker:
            return headWidthCollection.count
        case dpiPicker:
            return dpiCollection.count
        case stockWidthPicker:
            return stockWidthCollection.count
        case mediaTypePicker:
            return mediaTypeCollection.count
        case dispenseModePicker:
            return dispenseModeCollection.count
        default:
            
            return 0
        }
        

    }
    
    // returns the text to display on the row of the pickerview
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        
        switch pickerView {
            case headWidthPicker:
                return "\(headWidthCollection[row])"
            case dpiPicker:
                return "\(dpiCollection[row])"
            case stockWidthPicker:
                return "\(stockWidthCollection[row])"
            case mediaTypePicker:
                return "\(mediaTypeCollection[row])"
            case dispenseModePicker:
                return "\(dispenseModeCollection[row])"
            
            default:
                
                return "none"
        }
    }
    
    // sets the textfield contents after the pickerview has returned a selection
    func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        
        
        switch pickerView {
            case headWidthPicker:
                headWidthTextField.text = "\(headWidthCollection[row])"
            case dpiPicker:
                dpiTextField.text = "\(dpiCollection[row])"
            case stockWidthPicker:
                stockWidthTextField.text = "\(stockWidthCollection[row])"
            case mediaTypePicker:
                mediaTypeTextField.text = "\(mediaTypeCollection[row])"
            case dispenseModePicker:
                dispenseModeTextField.text = "\(dispenseModeCollection[row])"
            default:
                
                break
        }
        
        self.view.endEditing(false)
    }
    
    
    
    
    /**********************************************/
    // MARK: Helper functions
    /**********************************************/
    
    // this function simply populates the controls with current user preference values saved on the OS
    func initializeViewsWithPreferenceValues() {
        
        udpDiscoverySwitch.setOn(UserDefaults.standard.bool(forKey: "udpDiscovery"), animated: true)
        headWidthTextField.text = "\(UserDefaults.standard.integer(forKey: "printWidth"))"
        dpiTextField.text = "\(UserDefaults.standard.integer(forKey: "printerDPI"))"
        maintainAspectRatioSwitch.setOn(UserDefaults.standard.bool(forKey: "maintainAspectRatio"), animated: true)
        scaleToStockSwitch.setOn(UserDefaults.standard.bool(forKey: "scaleImageToFitStock"), animated: true)
        stockWidthTextField.text = UserDefaults.standard.string(forKey: "stockWidth")
        mediaTypeTextField.text = UserDefaults.standard.string(forKey:"mediaType")
        dispenseModeTextField.text = UserDefaults.standard.string(forKey:"dispenseMode")
        
        // number pad text fields
        advanceDistanceTextField.text = "\(UserDefaults.standard.integer(forKey: "advanceRetractDistance"))"
        advanceDistanceTextField.clearsOnBeginEditing = false
        
        advanceRetractDelayTextField.text = "\(UserDefaults.standard.integer(forKey: "advanceRetractDelay"))"
        advanceRetractDelayTextField.clearsOnBeginEditing = false

        stepsPastGapTextField.text = "\(UserDefaults.standard.integer(forKey: "stepsPastGap"))"
        stepsPastGapTextField.clearsOnBeginEditing = false

        activateGapDetectionTextField.text = "\(UserDefaults.standard.integer(forKey: "activateGapDetection"))"
        activateGapDetectionTextField.clearsOnBeginEditing = false

        drmTextField.text = "\(UserDefaults.standard.integer(forKey: "distanceToRegistrationOffset"))"
        drmTextField.clearsOnBeginEditing = false

        dotToGapTextField.text = "\(UserDefaults.standard.integer(forKey: "dotToGap_BlackLine"))"
        dotToGapTextField.clearsOnBeginEditing = false

        blacklineBlowholeHeightTextField.text = "\(UserDefaults.standard.integer(forKey: "blacklineBlowholeHeight"))"
        blacklineBlowholeHeightTextField.clearsOnBeginEditing = false

        gapHeightTextField.text = "\(UserDefaults.standard.integer(forKey: "gapHeight"))"
        gapHeightTextField.clearsOnBeginEditing = false

        
        // move the pickers to select the saved value
        self.headWidthPicker.selectRow(headWidthCollection.index(of: UserDefaults.standard.integer(forKey: "printWidth"))!, inComponent: 0, animated: false)
        self.dpiPicker.selectRow(dpiCollection.index(of: UserDefaults.standard.integer(forKey: "printerDPI"))!, inComponent: 0, animated: false)
        self.stockWidthPicker.selectRow(stockWidthCollection.index(of: UserDefaults.standard.string(forKey: "stockWidth")!)!, inComponent: 0, animated: false)
        self.mediaTypePicker.selectRow(mediaTypeCollection.index(of: UserDefaults.standard.string(forKey: "mediaType")!)!, inComponent: 0, animated: false)
        self.dispenseModePicker.selectRow(dispenseModeCollection.index(of: UserDefaults.standard.string(forKey: "dispenseMode")!)!, inComponent: 0, animated: false)
    }
    
    // this function sets the delegate and data source of the pickers, then adds the pickers as the inputview for the corresponding textfields
    func configurePickersForInput() {
        
        dpiPicker.delegate = self
        dpiPicker.dataSource = self
        dpiPicker.showsSelectionIndicator = true
        dpiTextField.inputView = dpiPicker
        dpiTextField.delegate = self
        
        headWidthPicker.delegate = self
        headWidthPicker.dataSource = self
        headWidthPicker.showsSelectionIndicator = true
        headWidthTextField.inputView = headWidthPicker
        headWidthTextField.delegate = self
        
        stockWidthPicker.delegate = self
        stockWidthPicker.dataSource = self
        stockWidthPicker.showsSelectionIndicator = true
        stockWidthTextField.inputView = stockWidthPicker
        stockWidthTextField.delegate = self
        
        mediaTypePicker.delegate = self
        mediaTypePicker.dataSource = self
        mediaTypePicker.showsSelectionIndicator = true
        mediaTypeTextField.inputView = mediaTypePicker
        mediaTypeTextField.delegate = self
        
        dispenseModePicker.delegate = self
        dispenseModePicker.dataSource = self
        dispenseModePicker.showsSelectionIndicator = true
        dispenseModeTextField.inputView = dispenseModePicker
        dispenseModeTextField.delegate = self
    }
    
    
    // this function is meant to clean up the viewDidLoad function, where all we do is register the "settingsValuesChanged" as a target for every control that changes a user setting.
    func addTargetToControls(){
        udpDiscoverySwitch.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .primaryActionTriggered)
        maintainAspectRatioSwitch.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .primaryActionTriggered)
        scaleToStockSwitch.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .primaryActionTriggered)
        headWidthTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        dpiTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        stockWidthTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        mediaTypeTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        dispenseModeTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        advanceDistanceTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        advanceRetractDelayTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        dotToGapTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        stepsPastGapTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        activateGapDetectionTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        drmTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        blacklineBlowholeHeightTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
        gapHeightTextField.addTarget(self, action: #selector(SettingsTableViewController.settingsValuesChanged(_:)), for: .editingDidEnd)
    }
}
