//
//  TCPSocks.swift
//  MicrocomFirstApp
//
//  Created by Cameron Wrabel on 6/14/17.
//  Copyright © 2017 Cameron Wrabel. All rights reserved.
//

import Foundation
import CoreFoundation

class TCPSocks: NSObject, StreamDelegate{
    
    //static var sharedInstance = TCPSocks() // singleton declaration
    
    var Port: UInt32
    var IP: String
    var InStream: InputStream?
    var OutStream: OutputStream?
    
    var hasStarted : Int = 0
    var hasConnected : Int = 0
    var isConnected : Int = 0
    
    var OutputString = ""
    var BufferSize = 1024
    var InStreamEvent: UInt32?
    var OutStreamEvent: UInt32?
    
    
//    var updateTextField: (()->Void)?
//    var readFromServerString: String {
//        didSet {
//            updateTextField?()
//        }
//    }
    
    init(port: UInt32, ip: String) {
        //readFromServerString = ""
        self.Port = port
        self.IP = ip
    }
    
    func startTCPClient(){
        
        var readStream : Unmanaged<CFReadStream>?
        var writeStream : Unmanaged<CFWriteStream>?
        let host : CFString = NSString(string: IP)
        let port : UInt32 = UInt32(Port)
        
        CFStreamCreatePairWithSocketToHost(kCFAllocatorDefault, host, port, &readStream, &writeStream)
        
        InStream = readStream!.takeRetainedValue()
        OutStream = writeStream!.takeRetainedValue()
        
        
        
        InStream!.delegate = self
        OutStream!.delegate = self
        
        InStream!.schedule(in: RunLoop.current, forMode: RunLoopMode.defaultRunLoopMode)
        OutStream!.schedule(in: RunLoop.current, forMode: RunLoopMode.defaultRunLoopMode)
        
        InStream!.open()
        OutStream!.open()
        
        hasStarted = 1
        isConnected = 1
        hasConnected = 1
    }
    
    // called when the hasBytesAvailable stream event is triggered
    func readFromServer() -> String{
        
        //OutputString = ""
        var buffer = [UInt8](repeating: 0, count: BufferSize )
        while(self.InStream!.hasBytesAvailable) {
            let bytesRead: Int = InStream!.read(&buffer, maxLength: buffer.count)
            if bytesRead >= 0 {
                OutputString += NSString(bytes: UnsafePointer(buffer), length: bytesRead, encoding: String.Encoding.ascii.rawValue)! as String
            } else {
                print(" ### Stream read error \n")
            }
        }
        
        //ViewController.readFromServerString = OutputString
        return OutputString
    }
    
    
    func writeToServer(message: String) -> Int {
        let encodedS = [UInt8]("\(message)\n".utf8) // add a newline
        let writeResult = OutStream!.write(encodedS, maxLength: encodedS.count)
        
        return writeResult
    }
    
    
    // this function does NOT append a CR/LF to the byte array, unlike the @func writeToServer
    func writeBytesToStream(bytes: [UInt8]) -> Int {
        let writeResult = OutStream!.write(bytes, maxLength: bytes.count)
        return writeResult
    }
    
    public func getStringReceived() -> String{
        let returnVal: String =  OutputString // temporarily store the string
        //OutputString = "" // reset the string so we don't read the same value more than once
        return returnVal
        
        
        
    }
    
    
    // event handler for both input and output streams. they are scheduled in the runloop,
    // and this class is an extension of StreamDelegate, so this class is the one notified on these event triggers
    func stream(_ stream: Stream, handle eventCode: Stream.Event){
        
        if stream === self.InStream {
            
            InStreamEvent = UInt32(eventCode.rawValue)
            
            switch eventCode {
            case Stream.Event.errorOccurred:
                print(" *** Stream Event: InStream: errorOccurred")
                isConnected = 0
                break
            case Stream.Event.openCompleted:
                print(" *** Stream Event: InStream: openCompleted")
                hasConnected = 1
                isConnected = 1
                break
            case Stream.Event.hasBytesAvailable:
                print(" *** Stream Event: InStream: hasBytesAvailable")
                //ViewController.readFromServerString = readFromServer()
                readFromServer()
                
                break
            default:
                break
                
            }
        } else if stream === self.OutStream {
            
            OutStreamEvent = UInt32(eventCode.rawValue)
            
            switch eventCode {
            case Stream.Event.errorOccurred:
                print(" *** Stream Event: OutStream: errorOccurred")
                isConnected = 0
                break
            case Stream.Event.openCompleted:
                print(" *** Stream Event: OutStream: openCompleted")
                hasConnected = 1
                isConnected = 1
                break
            case Stream.Event.hasSpaceAvailable:
                print(" *** Stream Event: OutStream: hasSpaceAvailable")
                break
            default:
                break
            }
        }
    }
    
    
    
    func closeSocks(){
        self.InStream?.close()
        self.OutStream?.close()
        
        isConnected = 0
        
    }
}
